"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createSiteModelFromTemplate = void 0;
const hub_common_1 = require("@esri/hub-common");
const hub_teams_1 = require("@esri/hub-teams");
const get_portal_site_hostname_1 = require("./get-portal-site-hostname");
const get_portal_site_url_1 = require("./get-portal-site-url");
const hub_common_2 = require("@esri/hub-common");
const interpolate_site_1 = require("./interpolate-site");
/**
 * Convert a Site Template into a Site Model
 * This will create Hub Teams and an Initiative, depending on licensing
 * and privs.
 *
 * The subdomain for the site will be constructed from the `settings.solution.title`
 * unless that contains unicode chars. In that case the initial subdomain will be `site`
 * and `ensureUniqueDomainName` will increment it as necessary (i.e. site-1, site-2 etc)
 *
 * This returns the Model that still needs to be saved!
 * @param {object} template Site Template
 * @param {object} settings Adlib interpolation hash
 * @param {object} transforms hash of transform functions
 * @param {IHubRequestOptions} hubRequestOptions
 */
function createSiteModelFromTemplate(template, settings, transforms, hubRequestOptions) {
    // add url to the assets, ref'ing the original location
    template.assets = hub_common_1.addSolutionResourceUrlToAssets(template, hubRequestOptions);
    // We may have templates which lack .properties so let's ensure that exists
    if (!template.item.properties) {
        template.item.properties = {};
    }
    // Kill props we don't want to roll forward if they happen to exist in the template
    [
        "customHostname",
        "externalUrl",
        "contentGroupId",
        "followersGroupId",
        "collaborationGroupId",
        "parentInitiativeId",
    ].forEach((prop) => {
        delete template.data.values[prop];
    });
    const product = hub_common_1.getHubProduct(hubRequestOptions.portalSelf);
    let title = hub_common_1.getProp(settings, "solution.title") || "New Site";
    // handle issue with titles that are just numbers
    if (typeof title === "number") {
        title = title.toString();
        hub_common_1.deepSet(settings, "solution.title", title);
    }
    // We need to carry some state through the promise chains
    // so we initialize an object outside the chain
    const state = {};
    // TODO: Eventually we'd like Enums
    const teamsToCreate = ["core", "content", "followers"];
    return hub_teams_1.createHubTeams({
        title,
        types: teamsToCreate,
        hubRequestOptions,
    })
        .then((teams) => {
        // fold teams into the settings hash - used mainly for cards
        settings.teams = hub_common_1.cloneObject(teams.props);
        state.teams = teams;
        // directly set the teams into the template item as this ensures
        // the team props are always set vs relying on adlib vars to exist
        Object.assign(template.item.properties, teams.props);
        if (hub_common_1.getProp(teams, "props.contentGroupId")) {
            hub_common_1.deepSet(template, "data.catalog.groups", [teams.props.contentGroupId]);
        }
        // sites need unique domains names
        // We derive this from the title, unless the title has unicode chars
        // in which case we use `site`, and the `ensureUniqueDomainName` function
        // will increment that as needed - i.e. `site-23`
        let domainTitle = title;
        if (hasUnicodeChars(domainTitle)) {
            domainTitle = "site";
        }
        return hub_common_2.ensureUniqueDomainName(hub_common_1.slugify(domainTitle), hubRequestOptions);
    })
        .then((uniqueSubdomain) => {
        const portal = hubRequestOptions.portalSelf;
        // TODO: Revisit this if/when we do more site templates which we want to maintain their theme
        settings.solution.theme = hub_common_1.getOrgDefaultTheme(portal);
        // set site-specific settings properties...
        settings.solution.subdomain = uniqueSubdomain;
        // setup the url properties
        if (hubRequestOptions.isPortal) {
            settings.solution.defaultHostname = get_portal_site_hostname_1.getPortalSiteHostname(uniqueSubdomain, portal);
            settings.solution.url = get_portal_site_url_1.getPortalSiteUrl(uniqueSubdomain, portal);
        }
        else {
            const base = hub_common_1.stripProtocol(hub_common_1.getHubApiUrl(hubRequestOptions));
            settings.solution.defaultHostname = `${uniqueSubdomain}-${portal.urlKey}.${base}`;
            settings.solution.url = `https://${uniqueSubdomain}-${portal.urlKey}.${base}`;
        }
    })
        .then((_) => {
        const siteModel = interpolate_site_1.interpolateSite(template, settings, transforms);
        // Special logic for the site title
        // if the title is a string, containing only numbers, then the interpolation will set it as
        // a number, which causes some problems... in that case, we stomp it in as a string...
        if (typeof siteModel.item.title === "number") {
            siteModel.item.title = hub_common_1.getProp(settings, "solution.title");
            siteModel.data.values.title = hub_common_1.getProp(settings, "solution.title");
        }
        return siteModel;
    })
        .catch((ex) => {
        throw Error(`site-utils::createSiteModelFromTemplate Error ${ex}`);
    });
}
exports.createSiteModelFromTemplate = createSiteModelFromTemplate;
/**
 * From Stackoverflow
 * https://stackoverflow.com/questions/147824/how-to-find-whether-a-particular-string-has-unicode-characters-esp-double-byte
 * This is the highest performance solution, combining three approaches
 */
const unicodeCharRegex = /[^\u0000-\u00ff]/;
function hasUnicodeChars(value) {
    if (value.charCodeAt(0) > 255)
        return true;
    return unicodeCharRegex.test(value);
}
//# sourceMappingURL=create-site-model-from-template.js.map